import 'package:adeptarch/adeptarch.dart';
import 'package:adeptarch/src/api/graphql_service.dart';
import 'package:adeptarch/src/api/query_response.dart';
import 'package:adeptutils/adeptutils.dart';
import 'package:graphql_flutter/graphql_flutter.dart';
import 'package:http/http.dart' as http;
import 'package:adeptarch/src/api/graphql_api.dart' as GraphQLAPI;
import 'dart:async';

/// Created by krrigan on 04/28/20.

abstract class GraphQLAPIService<I extends GraphQLService, DataType extends Jsonable, ErrorType>
{
  I service;
  DataType dataType;
  ErrorType errorType;
  GraphQLAPI.GraphQLAPI graphQLAPI;
  QueryResponseConverter<DataType> queryResponseConverter;

  GraphQLAPIService(this.service, {
    this.dataType,
    this.errorType,
    this.queryResponseConverter,
    this.graphQLAPI}) {
    graphQLAPI ??= GraphQLAPI.graphQLAPI;
    service?.graphQLClient ??= graphQLAPI?.client;
    queryResponseConverter ??= QueryResponseConverter<DataType>(data: dataType);
  }

  dynamic getAuthorization();

  updateHttpClient(http.Client httpClient){
    if(httpClient != null) graphQLAPI.init(httpClient: httpClient);
  }

  Future<QueryResult> getSaveResponse(Future<QueryResult> futureResponse) async{
    if(futureResponse == null) return null;
    QueryResult response;
    try{
        response = await futureResponse;
    } catch(e){
        if(e is QueryResult) response = e; else throw e;
    }
    return response;
  }

  Future<QueryResponse<DataType>> parseResponse(Future<QueryResult> futureResponse, DataTypeResponseConverter<DataType> dataTypeResponseConverter) async{
    return genericParseResponse(futureResponse, dataType, dataTypeResponseConverter);
  }

  Future<QueryResponse<List<DataType>>> parseResponseAsList(Future<QueryResult> futureResponse, DataTypeResponseConverter<List<DataType>> dataTypeResponseConverter) async{
    return genericParseResponseAsList(futureResponse, dataType, dataTypeResponseConverter);
  }

  Future<QueryResponse<DataType>> genericParseResponse<DataType extends Jsonable>(Future<QueryResult> futureResponse, DataType dataType, DataTypeResponseConverter<DataType> dataTypeResponseConverter) async {
    QueryResult queryResult = await getSaveResponse(futureResponse);
    QueryResponse<DataType> response;
    try{
      response = QueryResponse.fromMapQueryResult(queryResult, dataType, dataTypeResponseConverter);
    }catch(e){
      response = QueryResponse<DataType>(
        data: null,
        exception: queryResult?.exception,
        source: queryResult?.source,
        timestamp: queryResult.timestamp,
      );
      print(e);
    }
    return response;
  }

  Future<QueryResponse<List<DataType>>> genericParseResponseAsList<DataType extends Jsonable>(Future<QueryResult> futureResponse, DataType dataType, DataTypeResponseConverter<List<DataType>> dataTypeResponseConverter) async {
    QueryResult queryResult = await getSaveResponse(futureResponse);
    QueryResponse<List<DataType>> response;
    try{
      response = QueryResponse.fromListQueryResult(queryResult, dataType, dataTypeResponseConverter);
    }catch(e){
      response = QueryResponse<List<DataType>>(
        data: null,
        exception: queryResult?.exception,
        source: queryResult?.source,
        timestamp: queryResult.timestamp,
      );
      print(e);
    }
    return response;
  }
}
